#import "/global.typ": *
#pagebreak()

= How To Unity
Unity is a proprietary engine that might be used at no cost for small projects (see the #link("https://unity.com/legal/eula")[EULA] and whatever you have to consent, when downloading a license key).
It only officially supports C\# as scripting language.
Other .NET language might work, but I discourage using them,
and recommend sticking to the only language that is well tested, where bugs are fixed,
and that is used in all Unity documentation.

T obtain Unity, use #link("https://docs.unity3d.com/hub/manual/InstallHub.html")[UnityHub].
You will also need to #link("https://unity.com/products")[register as student or personal user].

Using Git with Unity is generally a nightmare, because Unity uses UUIDs in its files, making them difficult to read,
and randomly changes some fractional digits when different systems load a file and save it unchanged.
Use #link("https://github.com/github/gitignore/blob/main/Unity.gitignore")[this gitignore as a starting point],
to get some control over the giant pile of files Unity will create.

== UI
The GalacticKittens sample project referenced below still uses the old UI system.
It is simple to start with, but inflexible and unresponsive.
The new system #link("https://docs.unity3d.com/Manual/UIElements.html")[UI Toolkit] builds UIs from XML and CSS with #link("https://css-tricks.com/snippets/css/a-guide-to-flexbox/")[flexbox] for layouts.

#figure(```xml
<ui:UXML
  xmlns:ui="UnityEngine.UIElements" xmlns:uie="UnityEditor.UIElements"
  xsi="http://www.w3.org/2001/XMLSchema-instance"
  engine="UnityEngine.UIElements" editor="UnityEditor.UIElements"
  noNamespaceSchemaLocation="../UIElementsSchema/UIElements.xsd"
  editor-extension-mode="False"
>
  <ui:Style src="OverlayBig.uss"/>
  <ui:VisualElement name="full-screen-column">
      <ui:VisualElement name="top-row">
          <ui:Label class="fixed-portion" text="Testgame" name="mode"/>
          <ui:Label class="fixed-portion" text="Left" name="team-left"/>
          <ui:Label class="fixed-portion" text="" name="score"/>
          <ui:Label class="fixed-portion" text="Right" name="team-right"/>
          <ui:Label class="fixed-portion" text="0:00" name="time"/>
      </ui:VisualElement>
  </ui:VisualElement>
</ui:UXML>
```, caption: [UXML example])

#figure(```css
#top-row {
    flex-grow: initial;
    flex-direction: row;
    justify-content: center;
    font-size: 36;
    -unity-font: url("../Fonts/OpenSans-Bold.ttf");
    background-color: lightgray;
    height: 5%;
    /* vertical horizontal */
    padding: 2px 8px;
    margin: 0;
}
```, caption: [USS example])

// == Music and sound effects
// See #link("https://github.com/UnityTechnologies/GalacticKittens/blob/main/Assets/Scripts/Managers/AudioManager.cs")[the AudioManager from the GalacticKittens sample game] for sound management under Unity.

== Input processing
There are two input systems.
The old one, called #link("https://docs.unity3d.com/Manual/class-InputManager.html")[Input Manager], and the new one in the #link("https://docs.unity3d.com/Packages/com.unity.inputsystem@1.7/manual/index.html")[InputSystem-Package].
The new input system has a simple way (actionmaps) and a complex way (event handling) of processing user input.

The simple way is shown by #link("https://docs.unity3d.com/Packages/com.unity.inputsystem@1.7/manual/ActionAssets.html")[the docs].
You create an `*.inputactions` file with actions that are then mapped to `On<actionname>` methods.
This works if you have only one input device per local game instance.
However, it is unsuited if you want to allow a multiplayer mode by connecting multiple gamepads to the same PC.
In that case you need the to handle InputEvents, that know what device triggered them.

The complex way, used in that case, is to register handlers for events in `UnityEngine.InputSystem.InputSystem`.
#figure(```cs
// on device connect or disconnect
void OnDeviceChange(InputDevice device, InputDeviceChange change);
// on input
void OnInputEvent(InputEventPtr inputEvent, InputDevice inputDevice);

// register
UnityEngine.InputSystem.InputSystem.onDeviceChange += OnDeviceChange;
UnityEngine.InputSystem.InputSystem.onEvent += OnInputEvent;

// unregister
UnityEngine.InputSystem.InputSystem.onDeviceChange -= OnDeviceChange;
UnityEngine.InputSystem.InputSystem.onEvent -= OnInputEvent;
```, caption: [Usage of InputSystem Package])

#figure(image("/images/Unity/actionmaps.png"), caption: [
  Example of an action map with an action `Move` of type 2D vector,
  that can be triggered on keyboard by WASD keys or with on gamepad with the left stick.
])

== Networking
Use #link("https://github.com/Unity-Technologies/com.unity.netcode.gameobjects/tree/develop")[NetCode GameObjects].
You can see it in use #link("https://github.com/UnityTechnologies/GalacticKittens/tree/main")[in GalacticKittens].
See especially the `NetworkManager` component in the `Scenes/Bootstrap`, which has a list of Prefabs that need synchronization over the network.
See also the `Network Object`, `Client Network Transform`, and `Network RigidBody` components in `Prefabs/Net/Player/PlayerShipBase`, which are needed to determine what properties should be synchronized.

== Global state / Keep state across scenes
Keep entities with `DontDestroyOnLoad` after scene and initialize them in a bootstrapping scene.
Again, see #link("https://github.com/UnityTechnologies/GalacticKittens/blob/main/Assets/Scripts/Utility/Singleton.cs")[GalacticKittens].

== How to cross-compile?
You can adapt #link("https://osak.fsmpi.rwth-aachen.de/files/gamejam-2024-07-05/Builds.cs")[the build file from my bubbleball project]
and put it into `Assets/Editor/Builds.cs` to create a Unity Toolbar Item for builds.

To make it work, you need to have the modules for all supported platforms installed via UnityHub as seen in @unity-export.

#figure(image("/images/Unity/crosscompile.png"), caption: [
  Module options on Linux for Linux, macOS, and Windows support for both graphical and headless (dedicated server) mode.
  Since this screenshot was done on Linux, there is no `Linux Build Support (Mono)` option, which is built-in on Linux.
  If you are on Windows, the `Windows Build Support (Mono)` option would be built-in and `Linux Build Support (Mono)` has to be selected additionally.
]) <unity-export>